/* @file.c - ansi type buffered file functions built on top of bopen functions
 */
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include "gen.h"
#include "bopen.h"
#ifdef linux
#define __file _fileno
#endif

/* @getoflag - convert fopen type flags to open type flags
 */
int getoflag(char *type)
	{
	int oflag = O_BUFFER;

	if (strchr(type, '+'))
		oflag |= O_RDWR;
	else if (strchr(type, 'r'))
		oflag |= O_RDONLY;
	else
		oflag |= O_WRONLY;
	if (strchr(type, 'w'))
		oflag |= (O_TRUNC | O_CREAT);
	if (strchr(type, 'a'))
		oflag |= (O_APPEND | O_CREAT);
	return oflag;
	}

/* f@fopen
 */
FILE *zfopen(char *path, char *type)
	{
	FILE *rtn = NULL;
	int oflag;

	oflag = getoflag(type);
	if ((rtn = (FILE *)malloc(sizeof(FILE))) == NULL)
		;
	else if ((rtn->__file = bopen(path, oflag, 0777, &filebufmap)) < 0)
		free(rtn);
	return rtn;
	}

/* f@fclose
 */
int zfclose(FILE *file)
	{
	int rtn;

	rtn = bclose(file->__file);
	free(file);
	return rtn;
	}

/* f@fread
 */
int zfread(char *ptr, size_t size, int nitems, FILE *stream)
	{
	int rtn;

	errno = 0;
	if (0 <= (rtn = bread(stream->__file, ptr, size * nitems)))
		rtn = rtn / size;
	else
		rtn = 0;
	return rtn;
	}

/* f@fwrite
 */
int zfwrite(char *ptr, size_t size, int nitems, FILE *stream)
	{
	int rtn;

	errno = 0;
	if (0 <= (rtn = bwrite(stream->__file, ptr, size * nitems)))
		rtn = rtn / size;
	else
		rtn = 0;
	return rtn;
	}

/* f@fgets
 */
char *zfgets(char *s, int n, FILE *stream)
	{
	char *rtn = NULL;

	errno = 0;
	if (0 < get(stream->__file, s, n))
		rtn = s;
	return rtn;
	}

/* f@fputs
 */
int zfputs(char *s, FILE *stream)
	{
	return bwrite(stream->__file, s, strlen(s));
	}

/* f@fflush
 */
int zfflush(FILE *stream)
	{
	return flush(stream->__file);
	}
