/* @xecv.c - execute a file with argument vector
 */
#include <unistd.h>
#include <errno.h>
#include <signal.h>
#include "gen.h"
#include "xecv.h"

#define COREDUMP 	0x80

static char *errlist[] = {
					"unknown signal",
					"hangup",
					NULL,
					"quit",
					"illegal instruction",
					"trace trap",
					"range error",
					"domain error",
					"floating point trap",
					"kill",
					"bus error",
					"segmentation error",
					"system call error",
					"broken pipe",
					"alarm",
					NULL
					};

#define NUMSIG 15


/* @xecv - execute a program
 */
int xecv(char *fname, int sin, int sout, short flags, char **av)
	{
	register short pid;
	int pstat;

#	ifdef DEB
	errfmt("xecv: fname '%s' sin %d sout %d flags 0%o *av '%p'\n",
		fname, sin, sout, flags, *av);
#	endif
	if (!(flags & X_OVLAY) && (pid = fork()) != 0)
		{
		int signum;
		/* parent */
		if (sout != STDOUT_FILENO)
			close(sout);
		if (sin != STDIN_FILENO)
			close(sin);
		if (pid < 0)
			{
			eprint("%s: %s: %s\n", progname, fname, errmesg(errno));
			return (-1);
			}
		else if (flags & X_ASYN)
			return (pid);
		else
			{
#			ifdef DEB
			errfmt("xecv: parent waiting\n");
#			endif
			while (wait(&pstat) != pid)
				;
#			ifdef DEB
			errfmt("xecv: pstat %d\n", pstat);
#			endif
			if (!(pstat & HIBYTE) && (pstat & LOBYTE))
				{
				signum = pstat & 0x7f;
				signum = (signum < NUMSIG ? signum : 0);
				if (errlist[signum] != NULL)
					eprint("%s: signal %d - %s %s\n", *av, pstat & 0x7f, errlist[signum], 
						(pstat & COREDUMP) ? "- core dumped\n":"");
				}
#			ifdef DEB
			errfmt("xecv: here pstat %d\n", (pstat >> 8)  & LOBYTE| (pstat << 8) &HIBYTE);
#			endif
			return ((pstat >> 8) & LOBYTE | (pstat << 8) & HIBYTE);
			}
		}
	else
		{
		/* child */
		if (flags & X_SYSIG)
			{
			signal(SIGINT, NULL);
			signal(SIGQUIT, NULL);
			}
		if (flags & X_RUID)
			setuid(getuid());
		if (sout != STDOUT_FILENO)
			{
			close(STDOUT_FILENO);
			dup2(sout, STDOUT_FILENO);
			close(sout);
			}
		if (sin != STDIN_FILENO)
			{
			close(STDIN_FILENO);
			dup2(sin, STDIN_FILENO);
			close(sin);
			}
		execvpe(fname, av, NULL);
		eprint("%s: %s: %s\n", progname, fname, errmesg(errno));
		_exit(1);
		}
	}

#ifdef TEST
 int main(int argc, char *argv[])
	{
	exit(xecv(argv[1], STDIN_FILENO, STDOUT_FILENO, 0, &argv[1]));
	}
#endif
